@@ -5,7 +5,7 @@ package OpenAPI::Modern;
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI Swagger HTTP request response
 
-our $VERSION = '0.010';
+our $VERSION = '0.011';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -21,6 +21,7 @@ use Ref::Util qw(is_plain_hashref is_plain_arrayref is_ref);
 use List::Util 'first';
 use Scalar::Util 'looks_like_number';
 use Feature::Compat::Try;
+use Encode 2.89;
 use JSON::Schema::Modern 0.531;
 use JSON::Schema::Modern::Utilities 0.531 qw(jsonp unjsonp canonical_uri E abort);
 use JSON::Schema::Modern::Document::OpenAPI;
@@ -140,12 +141,12 @@ sub validate_request ($self, $request, $options) {
         $body_obj = $self->_resolve_ref($ref, $state);
       }
 
-      if ($body_obj->{required} and not length($request->content_ref->$*)) {
-        ()= E({ %$state, keyword => 'required' }, 'request body is required but missing');
-      }
-      else {
+      if ($request->content_length) {
         ()= $self->_validate_body_content($state, $body_obj->{content}, $request);
       }
+      elsif ($body_obj->{required}) {
+        ()= E({ %$state, keyword => 'required' }, 'request body is required but missing');
+      }
     }
   }
   catch ($e) {
@@ -500,7 +501,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI docume
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -557,11 +558,10 @@ version 0.010
   );
 
   say 'request:';
-  my $request = HTTP::Request->new(
-    POST => 'http://example.com/foo/bar',
+  use HTTP::Request::Common;
+  my $request = POST 'http://example.com/foo/bar',
     [ 'My-Request-Header' => '123', 'Content-Type' => 'application/json' ],
-    '{"hello": 123}',
-  );
+    '{"hello": 123}';
   say $openapi->validate_request($request, {
     path_template => '/foo/{foo_id}',
     path_captures => { foo_id => 'bar' },
